defmodule DecentralisedBookIndexWeb.DbiServerLive.FormComponent do
  use DecentralisedBookIndexWeb, :live_component

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
      </.header>

      <.simple_form
        for={@form}
        id="dbi_server-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:name]} type="text" label="Name" /><.input
          field={@form[:url]}
          type="text"
          label="Url"
        />

        <:actions>
          <.save_button phx-disable-with="Saving...">
            Save
          </.save_button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_form()}
  end

  @impl true
  def handle_event("validate", %{"dbi_server" => dbi_server_params}, socket) do
    {:noreply,
     assign(socket, form: AshPhoenix.Form.validate(socket.assigns.form, dbi_server_params))}
  end

  def handle_event("save", %{"dbi_server" => dbi_server_params}, socket) do
    case AshPhoenix.Form.submit(socket.assigns.form, params: dbi_server_params) do
      {:ok, dbi_server} ->
        notify_parent({:saved, dbi_server})

        socket =
          socket
          |> put_flash(:info, "Server #{socket.assigns.form.source.type}d successfully")
          |> push_patch(to: socket.assigns.patch)

        {:noreply, socket}

      {:error, form} ->
        {:noreply, assign(socket, form: form)}
    end
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})

  defp assign_form(%{assigns: %{dbi_server: dbi_server}} = socket) do
    form =
      if dbi_server do
        AshPhoenix.Form.for_update(dbi_server, :update,
          as: "dbi_server",
          actor: socket.assigns.current_user
        )
      else
        AshPhoenix.Form.for_create(DecentralisedBookIndex.Metadata.DBIServer, :create,
          as: "dbi_server",
          actor: socket.assigns.current_user
        )
      end

    assign(socket, form: to_form(form))
  end
end
